package bio.singa.chemistry.features.molarmass;

import bio.singa.chemistry.features.databases.chebi.ChEBIDatabase;
import bio.singa.chemistry.features.databases.uniprot.UniProtDatabase;
import bio.singa.features.identifiers.ChEBIIdentifier;
import bio.singa.features.identifiers.UniProtIdentifier;
import bio.singa.features.model.FeatureProvider;
import bio.singa.features.model.Featureable;
import bio.singa.structure.features.molarmass.MolarMass;

/**
 * @author cl
 */
public class MolarMassProvider extends FeatureProvider<MolarMass> {

    private final ChEBIDatabase chEBIDatabase = ChEBIDatabase.getInstance();
    private final UniProtDatabase uniProtDatabase = UniProtDatabase.getInstance();

    public MolarMassProvider() {
        setProvidedFeature(MolarMass.class);
        addRequirement(ChEBIIdentifier.class);
        addFallbackRequirement(100, UniProtIdentifier.class);
    }

    @Override
    public <FeatureableType extends Featureable> MolarMass provide(FeatureableType featureable) {
        // mass is parsed from databases
        switch (getPreferredStrategyIndex()) {
            case 0: {
                MolarMass molarMass = ChEBIDatabase.fetchMolarMass(featureable);
                if (molarMass != null) {
                    return molarMass;
                }
            }
            case 100: {
                MolarMass molarMass = UniProtDatabase.fetchMolarMass(featureable);
                if (molarMass != null) {
                    return molarMass;
                }
            }
        }
        return null;
    }
}
